package com.qmth.ops.biz.ai.client.aliyun.llm;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.qmth.boot.core.ai.model.llm.ChatRequest;
import com.qmth.boot.core.ai.model.llm.ChatResult;
import com.qmth.boot.core.exception.ReentrantException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.ops.biz.ai.client.ChatApiClient;
import com.qmth.ops.biz.ai.client.ChatApiConfig;
import com.qmth.ops.biz.ai.client.aliyun.AliyunError;
import com.qmth.ops.biz.ai.exception.ChatRequestError;
import okhttp3.Headers;

import java.io.IOException;

public class AliyunChatClient extends ChatApiClient {

    private static final String AUTH_HEADER_NAME = "Authorization";

    private static final String AUTH_HEADER_VALUE = "Bearer ";

    public AliyunChatClient(ChatApiConfig config) {
        super(config);
    }

    @Override
    protected Headers buildHeader(Headers.Builder headerBuilder) {
        return headerBuilder.add(AUTH_HEADER_NAME, AUTH_HEADER_VALUE + getConfig().getSecret())
                .add("X-DashScope-DataInspection", "{\"input\":\"disable\", \"output\":\"disable\"}").build();
    }

    @Override
    protected Object buildRequest(ChatRequest request) {
        return new AliyunChatRequest(request, getConfig().getModel());
    }

    @Override
    protected ChatResult buildResult(byte[] data, ObjectMapper mapper) throws IOException {
        return mapper.readValue(data, AliyunChatResult.class).buildResult();
    }

    @Override
    protected ChatResult handleError(byte[] data, int statusCode, ObjectMapper mapper) {
        AliyunError error = null;
        if (data != null) {
            try {
                error = mapper.readValue(data, AliyunError.class);
            } catch (Exception e) {
            }
        }
        switch (statusCode) {
        case 400:
            throw new ChatRequestError(error != null ? error.getMessage() : "chat request error");
        case 429:
            throw new ReentrantException(error != null ? error.getMessage() : "chat model rate limit exceeded");
        default:
            throw new StatusException(error != null ? error.getMessage() : "chat model error");
        }
    }

}
